
--   $ot_monitor_serverpb = q{

Create or replace package body ot_monitor_server is

/* Copyright 2002, Adam vonNieda */

   /* Globals */

   vDebug	NUMBER;

   Procedure debug_message

/* Write a message to the repository database alert log */

   (
      pText		IN VARCHAR2
   )
   IS

   BEGIN
      IF vDebug > 0 THEN
         sys.dbms_system.ksdwrt(2,'Oracletool: '||pText);
      END IF;
   END;

   Procedure checkall

/* 

   Loop through the clients, and invoke ot_monitor_client.checkall
   on each of them. 

*/

   (
      pDebug			IN NUMBER DEFAULT 0
   )

   IS

   vInstanceName	VARCHAR2(16);
   vDummy		pls_integer;
   linkcursor		pls_integer default dbms_sql.open_cursor;
   sessioncursor	pls_integer default dbms_sql.open_cursor;
   vSql			VARCHAR2(100);
   vSqlCode		NUMBER;
   vSqlErrm		VARCHAR2(4000);
   vErrorStack		VARCHAR2(4000);
   vEveryThingsOK	BOOLEAN;

   CURSOR getinstances IS
   SELECT CLIENT FROM OT_MONITOR_CLIENTS
   WHERE STATUS = 'Enabled';

   CURSOR debugCursor IS
   SELECT VALUE FROM OT_MONITOR_SETTINGS
   WHERE NAME = 'Debug';

      BEGIN

---------------------------------------------------------------------------------------------------------------------
-- Set GLOBAL_NAMES to FALSE so we can use our OT_MONITOR database link..
---------------------------------------------------------------------------------------------------------------------

      dbms_sql.parse(sessioncursor,'alter session set global_names = FALSE',dbms_sql.native);
      vDummy := dbms_sql.execute(sessioncursor);
      dbms_sql.close_cursor(sessioncursor);

---------------------------------------------------------------------------------------------------------------------
-- Set debug level. First, check for passed value, then check server.
---------------------------------------------------------------------------------------------------------------------

      IF pDebug > 0 THEN
         vDebug := pDebug;
      ELSE
         OPEN debugCursor;
         FETCH debugCursor INTO vDebug;
         CLOSE debugCursor;
      END IF;

      debug_message('Debug is set to '||vDebug);

-- Loop through the instance names that are 'Enabled'

      OPEN getinstances;

      LOOP
         FETCH getinstances INTO vInstanceName;
            exit when getinstances%NOTFOUND;
         debug_message('Invoking check for client '||vInstanceName);
         vSql := 'begin ot_monitor_client.checkall@'||vInstanceName||';end;';
         debug_message('Set vSql');

         BEGIN
            dbms_sql.parse(linkcursor,vSql,dbms_sql.native);
            debug_message('Parsed '||vInstanceName);
            debug_message('Executing '||vInstanceName);
            vDummy := DBMS_SQL.EXECUTE(linkcursor);
            EXCEPTION
               WHEN OTHERS THEN
                  debug_message('In exception '||vInstanceName);
                  vSqlCode := SQLCODE;
                  debug_message('Error: '||vSqlCode);
                  vErrorStack := DBMS_UTILITY.FORMAT_ERROR_STACK;
                  INSERT INTO OT_MONITOR_ALERTS (CLIENT,ALERT)
                     VALUES (vInstanceName,'Fatal error!: '||vSqlCode||' : '||vErrorStack);
                  vEveryThingsOK := FALSE;
                  debug_message('Executed '||vInstanceName);
         END;

         UPDATE OT_MONITOR_CLIENTS SET LAST_CHECKED = SYSDATE WHERE CLIENT = vInstanceName;
         COMMIT;
         debug_message('Commited '||vInstanceName);
      END LOOP;

      DBMS_SQL.CLOSE_CURSOR(linkcursor);
      CLOSE getinstances;
   END;

   Procedure process_alert

/*

   This procedure is invoked by the OT_MONITOR_ALERT_TRIG
   trigger, which is invoked when a record is inserted
   into the OT_MONITOR_ALERTS table. This proc will determine
   if an alert has already been sent for the particular
   alert and ignore it if it has. If the alert is different
   from one already sent, then it will end invoke the send_mail
   procedure.

*/

      (
         pInstanceName		IN VARCHAR2,
         pResourceType		IN VARCHAR2,
         pResourceName		IN VARCHAR2,
         pAlert			IN VARCHAR2,
         pDebug			IN NUMBER DEFAULT 0
      )

      IS

         vRecipient		VARCHAR2(50);
         vSubject		VARCHAR2(100);
         vMessage		VARCHAR2(4000);
         vAlert			VARCHAR2(4000);
         vSent			NUMBER;

      CURSOR already_sent IS
      SELECT
         COUNT(*)
      FROM OT_MONITOR_ALERTS_SENT
         WHERE CLIENT = pInstanceName
         AND RESOURCE_TYPE = pResourceType
         AND RESOURCE_NAME = pResourceName
         AND ALERT = pAlert;

      CURSOR instance_recipients IS
      SELECT
         EMAIL_ADDRESS
      FROM OT_MONITOR_RECIPIENTS
         WHERE CLIENT = pInstanceName;

      CURSOR global_recipients IS
      SELECT
         EMAIL_ADDRESS
      FROM OT_MONITOR_RECIPIENTS
         WHERE CLIENT = 'ALL';

      BEGIN

---------------------------------------------------------------------------------------------------------------------
-- See if an email has already been sent for this alert.
-- If one has, ignore this alert..
---------------------------------------------------------------------------------------------------------------------

         vAlert := pInstanceName||': '||pAlert;

         OPEN already_sent;

            FETCH already_sent INTO vSent;

         CLOSE already_sent;

         IF vSent > 0 THEN
            debug_message('An alert has already been sent for alert: '||vAlert);
         ELSE
            debug_message('Sending alert: '||vAlert);

            vSubject := SUBSTR(vAlert,1,100);
            vMessage := vAlert;

            -- Send to people who have selected the instance by name, not global recipients.

            open instance_recipients;
            loop
               fetch instance_recipients into vRecipient;
               exit when instance_recipients%NOTFOUND;

               send_mail(vRecipient,vSubject,vMessage);

               INSERT INTO OT_MONITOR_ALERTS_SENT
                  (CLIENT,RESOURCE_TYPE,RESOURCE_NAME,ALERT,RECIPIENT)
                  VALUES (pInstanceName,pResourceType,pResourceName,pAlert,vRecipient);
            end loop;

            -- Send to people who receive all alerts from all instances, global recipients.

            open global_recipients;
            loop
               fetch global_recipients into vRecipient;
               exit when global_recipients%NOTFOUND;

               send_mail(vRecipient,vSubject,vMessage);

               INSERT INTO OT_MONITOR_ALERTS_SENT
                  (CLIENT,RESOURCE_TYPE,RESOURCE_NAME,ALERT,RECIPIENT)
                  VALUES (pInstanceName,pResourceType,pResourceName,pAlert,vRecipient);
            end loop;

         END IF;

     END;

   Procedure send_mail

/*

   This procedure sends emails. 

*/

      (
         pRecipient		IN VARCHAR2,
         pSubject		IN VARCHAR2,
         pMessage		IN VARCHAR2,
         pDebug			IN NUMBER DEFAULT 0
      )

      IS

      vMailConn		utl_smtp.connection;
      vMailHost		VARCHAR2(100);
      vPort		NUMBER;
      vSender		VARCHAR2(50);
      vSent		NUMBER;
      vEveryThingsOK	BOOLEAN;
      vSqlCode		NUMBER;
      vErrorStack	VARCHAR2(4000);

      CURSOR debugCursor IS
      SELECT VALUE FROM OT_MONITOR_SETTINGS
      WHERE NAME = 'Debug';

      CURSOR mailserver IS
      SELECT VALUE FROM OT_MONITOR_SETTINGS WHERE NAME = 'Mailserver';

      CURSOR mailserverport IS
      SELECT TO_NUMBER(VALUE) FROM OT_MONITOR_SETTINGS WHERE NAME = 'Mailserverport';

      CURSOR mailsender IS
      SELECT VALUE FROM OT_MONITOR_SETTINGS WHERE NAME = 'Mailsender';

      BEGIN

---------------------------------------------------------------------------------------------------------------------
-- Set debug level. First, check for passed value, then check server.
---------------------------------------------------------------------------------------------------------------------

         IF pDebug > 0 THEN
            vDebug := pDebug;
         ELSE
            OPEN debugCursor;
            FETCH debugCursor INTO vDebug;
            CLOSE debugCursor;
         END IF;

         debug_message('Debug is set to '||vDebug);

---------------------------------------------------------------------------------------------------------------------
--   Set mailhost to the machine the database is on if
--   the mailhost is not specified.
---------------------------------------------------------------------------------------------------------------------

         OPEN mailserver;
            FETCH mailserver INTO vMailHost;
         CLOSE mailserver;

         OPEN mailsender;
            FETCH mailsender INTO vSender;
         CLOSE mailsender;

         OPEN mailserverport;
            FETCH mailserverport INTO vPort;
         CLOSE mailserverport;

         debug_message('Sending to '||pRecipient||' via '||vMailHost||' Port '||vPort);

         BEGIN

            vMailConn := utl_smtp.open_connection(vMailHost, vPort);

            utl_smtp.helo(vMailConn, vMailHost);

            utl_smtp.mail(vMailConn, vSender);

            utl_smtp.rcpt(vMailConn, pRecipient);

            utl_smtp.data(vMailConn,
               'Subject : '||pSubject || CHR(10) || CHR(13) || pMessage);

            utl_smtp.quit(vMailConn);

               EXCEPTION
                  WHEN OTHERS THEN
                     debug_message('   Proc send_mail: In exception.');
                     vSqlCode := SQLCODE;
                     debug_message('Error: '||vSqlCode);
                     vErrorStack := DBMS_UTILITY.FORMAT_ERROR_STACK;
                     INSERT INTO OT_MONITOR_ALERTS_SENT (CLIENT,ALERT,RECIPIENT)
                        VALUES ('ALL','Fatal mail server error!: '||vSqlCode||' : '||vErrorStack,pRecipient);
                        vEveryThingsOK := FALSE;
         END;

      END;

   END OT_MONITOR_SERVER;

--   };
